Subject: Re: [PATCH RFC] ipv6: fix route selection if kernel is not compiled with CONFIG_IPV6_ROUTER_PREF

From: Nicolas Dichtel <[email-masked]>
Date: Thu, 11 Jul 2013 10:04:47 +0200

Le 10/07/2013 23:21, Hannes Frederic Sowa a écrit :
> On Wed, Jul 10, 2013 at 04:10:58PM +0200, Nicolas Dichtel wrote:
>> I wonder why expires is 0. Even if this route is cached, the flag
>> RTF_EXPIRES should be set. Am I wrong?
>
> rt6_set_from deliberately clears the RTF_EXPIRES when creating a cached copy
> of the route if the route is an autoconfigured default route.
>
> Maybe the criterion for exclusion of which routes can get into an ecmp route
> set should be revisited? This could result in strange effects for users
> working with two interfaces, both receiving a RA with default routes.
Agreed. Here is a proposal, what do you think?

[PATCH] ipv6: don't use autoconfigured route for ecmp

The intention was already there by checking the flag RTF_EXPIRES, but this flag
is removed from the default route by rt6_set_from() when this route is cached.

Let's add a check against RTF_ADDRCONF.

Spotted-by: Hannes Frederic Sowa <[email-masked]>
Signed-off-by: Nicolas Dichtel <[email-masked]>
---
  net/ipv6/ip6_fib.c | 6 +++---
  1 file changed, 3 insertions(+), 3 deletions(-)

diff --git a/net/ipv6/ip6_fib.c b/net/ipv6/ip6_fib.c
index 192dd1a..87e31c6 100644
--- a/net/ipv6/ip6_fib.c
+++ b/net/ipv6/ip6_fib.c
@@ -686,14 +686,14 @@ static int fib6_add_rt2node(struct fib6_node *fn, struct 
rt6_info *rt,
  			 * of siblings, and later we will add our route to the
  			 * list.
  			 * Only static routes (which don't have flag
 			 * RTF_EXPIRES) are used for ECMPv6.
  			 *
  			 * To avoid long list, we only had siblings if the
  			 * route have a gateway.
  			 */
  			if (rt->rt6i_flags & RTF_GATEWAY &&
-			    !(rt->rt6i_flags & RTF_EXPIRES) &&
-			    !(iter->rt6i_flags & RTF_EXPIRES))
+			    !(rt->rt6i_flags & (RTF_EXPIRES|RTF_ADDRCONF)) &&
+			    !(iter->rt6i_flags & (RTF_EXPIRES|RTF_ADDRCONF)))
  				rt->rt6i_nsiblings++;
  		}
@@ -686,14 +686,14 @@ static int fib6_add_rt2node(struct fib6_node *fn, struct 
  			 * list.
  			 * Only static routes (which don't have flag
-			 * RTF_EXPIRES) are used for ECMPv6.
+			 * RTF_EXPIRES nor RTF_ADDRCONF) are used for ECMPv6.
  			 *
  			 * To avoid long list, we only had siblings if the

-- 
1.8.2.1